package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.HosRelationNumDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.ScaleConfig;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.ScaleConfigService;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.HosRelationNumPageVO;
import com.diagbot.vo.HospitalIdVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.ScaleConfigListVO;
import com.diagbot.vo.ScaleConfigPageVO;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/4/13 16:15
 */
@Component
public class ScaleConfigFacade {
    @Autowired
    private ScaleConfigService scaleConfigService;
    @Autowired
    private CdssCoreClient cdssCoreClient;

    /**
     * 判断是否已存在
     *
     * @param scaleConfig
     * @return
     */
    public Boolean isExistRecord(ScaleConfig scaleConfig) {
        QueryWrapper<ScaleConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", scaleConfig.getHospitalId())
                .eq("his_name", scaleConfig.getHisName())
                .eq("unique_name", scaleConfig.getUniqueName());
        ScaleConfig oldRecord = scaleConfigService.getOne(queryWrapper, false);
        if (scaleConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (scaleConfig.getId() != null
                && oldRecord != null
                && !scaleConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param scaleConfig
     * @return
     */
    public Boolean saveOrUpdateRecord(ScaleConfig scaleConfig) {
        String userId = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        scaleConfig.setModifier(userId);
        scaleConfig.setGmtModified(now);
        QueryWrapper<ScaleConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", scaleConfig.getHospitalId())
                .eq("his_name", scaleConfig.getHisName())
                .eq("unique_name", scaleConfig.getUniqueName());
        ScaleConfig oldRecord = scaleConfigService.getOne(queryWrapper, false);
        if (scaleConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (scaleConfig.getId() != null
                && oldRecord != null
                && !scaleConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        //新增数据
        if (scaleConfig.getId() == null) {
            scaleConfig.setCreator(userId);
            scaleConfig.setGmtCreate(now);
        }
        if (scaleConfig.getIsDeleted() == null) {
            scaleConfig.setIsDeleted(IsDeleteEnum.N.getKey());
        }
        scaleConfigService.saveOrUpdate(scaleConfig);
        return true;
    }

    /**
     * 保存记录-批量
     *
     * @param scaleConfigListVO
     * @return
     */
    public Boolean saveOrUpdateRecords(ScaleConfigListVO scaleConfigListVO) {
        if (ListUtil.isEmpty(scaleConfigListVO.getScaleConfigList())) {
            return false;
        }
        return saveOrUpdateRecords(scaleConfigListVO.getScaleConfigList());
    }

    /**
     * 批量保存
     *
     * @param scaleConfigList
     * @return
     */
    public Boolean saveOrUpdateRecords(List<ScaleConfig> scaleConfigList) {
        if (ListUtil.isEmpty(scaleConfigList)) {
            return false;
        }

        String userId = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //数据不完整的不保存
        //过滤外部名称或公表名为空的数据
        scaleConfigList = scaleConfigList
                .stream()
                .filter(i -> i.getHospitalId() != null)
                .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());

        if (ListUtil.isEmpty(scaleConfigList)) {
            return false;
        }

        Long hospitalId = scaleConfigList.get(0).getHospitalId();

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, List<Long>>> configMap
                = getConfigMap(hospitalId, null, null);
        scaleConfigList.forEach(scaleConfig -> {
            scaleConfig.setModifier(userId);
            scaleConfig.setGmtModified(now);
            if (scaleConfig.getId() == null) {
                if (configMap.containsKey(scaleConfig.getHisName())
                        && ListUtil.isNotEmpty(configMap.get(scaleConfig.getHisName()).get(scaleConfig.getUniqueName()))) {
                    deleteIds.addAll(configMap.get(scaleConfig.getHisName()).get(scaleConfig.getUniqueName()));
                }
                scaleConfig.setCreator(userId);
                scaleConfig.setGmtCreate(now);
            }
            if (scaleConfig.getIsDeleted() == null) {
                scaleConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });
        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        scaleConfigService.saveOrUpdateBatch(scaleConfigList);
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        UpdateWrapper<ScaleConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", idVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        scaleConfigService.removeById(idVO.getId());
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        UpdateWrapper<ScaleConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", idListVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        scaleConfigService.removeByIds(idListVO.getIds());
        return true;
    }

    /**
     * 分页查询
     *
     * @param scaleConfigPageVO
     * @return
     */
    public IPage<ScaleConfig> getPage(ScaleConfigPageVO scaleConfigPageVO) {
        return scaleConfigService.getPage(scaleConfigPageVO);
    }

    /**
     * 数据导入
     *
     * @param file
     * @param hospitalIdVO
     */
    public void importExcel(MultipartFile file, HospitalIdVO hospitalIdVO) {
        List<ScaleConfig> scaleConfigList = ExcelUtils.importExcel(file, 0, 1, ScaleConfig.class);
        if (ListUtil.isNotEmpty(scaleConfigList)) {
            scaleConfigList.forEach(scaleConfig -> {
                scaleConfig.setHospitalId(hospitalIdVO.getHospitalId());
            });
            importExcelRecords(scaleConfigList, hospitalIdVO);
        } else {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
    }

    /**
     * 数据导入
     *
     * @param scaleConfigList
     * @return
     */
    public Boolean importExcelRecords(List<ScaleConfig> scaleConfigList, HospitalIdVO hospitalIdVO) {
        Long hospitalId = hospitalIdVO.getHospitalId();
        String userId = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //1、数据完整性校验
        //2、去除前后空格
        //过滤空数据，保留重复数据，方便计行
        scaleConfigList = scaleConfigList.stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName())
                        || StringUtil.isNotBlank(i.getUniqueCode())
                        || StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(scaleConfigList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
        List<String> emptyNumList = Lists.newLinkedList();
        for (int i = 0; i < scaleConfigList.size(); i++) {
            if (StringUtil.isBlank(scaleConfigList.get(i).getHisName())
                    || StringUtil.isBlank(scaleConfigList.get(i).getUniqueName())) {
                emptyNumList.add(String.valueOf(i + 2));
            }
            if (StringUtil.isNotBlank(scaleConfigList.get(i).getHisName())) {
                scaleConfigList.get(i).setHisName(scaleConfigList.get(i).getHisName().trim());
            }
            if (StringUtil.isNotBlank(scaleConfigList.get(i).getUniqueName())) {
                scaleConfigList.get(i).setUniqueName(scaleConfigList.get(i).getUniqueName().trim());
            }
            if (StringUtil.isNotBlank(scaleConfigList.get(i).getUniqueCode())) {
                scaleConfigList.get(i).setUniqueCode(scaleConfigList.get(i).getUniqueCode().trim());
            } else {
                scaleConfigList.get(i).setUniqueCode(null);
            }
        }

        if (ListUtil.isNotEmpty(emptyNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "以下行数（不计入空行）存在不完整数据："
                    + emptyNumList.stream().collect(Collectors.joining("、"))
                    + "。导入取消，请修改后再试。\n");
        }

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, List<Long>>> configMap
                = getConfigMap(Long.valueOf(hospitalId), null, null);
        scaleConfigList.forEach(scaleConfig -> {
            scaleConfig.setHospitalId(Long.valueOf(hospitalId));
            scaleConfig.setModifier(userId);
            scaleConfig.setGmtModified(now);
            if (scaleConfig.getId() == null) {
                if (configMap.containsKey(scaleConfig.getHisName())
                        && ListUtil.isNotEmpty(configMap.get(scaleConfig.getHisName()).get(scaleConfig.getUniqueName()))) {
                    deleteIds.addAll(configMap.get(scaleConfig.getHisName()).get(scaleConfig.getUniqueName()));
                }
                scaleConfig.setCreator(userId);
                scaleConfig.setGmtCreate(now);
            }
            if (scaleConfig.getIsDeleted() == null) {
                scaleConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });

        //标准术语校验
        List<String> errorNumList = Lists.newLinkedList();
        List<String> uniqueNames = scaleConfigList.stream()
                .map(i -> i.getUniqueName())
                .distinct()
                .collect(Collectors.toList());
        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setNames(uniqueNames);
        conceptVO.setType(ConceptTypeEnum.Scale.getKey());
        RespDTO<List<String>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        List<String> names = respDTO.data;
        for (int i = 0; i < scaleConfigList.size(); i++) {
            if (!names.contains(scaleConfigList.get(i).getUniqueName())) {
                errorNumList.add(String.valueOf(i + 2));
            }
        }
        if (ListUtil.isNotEmpty(errorNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "以下行数（不计空行）标准术语在数据库中不存在："
                            + errorNumList.stream().collect(Collectors.joining("、"))
                            + "。导入取消，请修改后再试。");
        }

        //重复数据过滤
        scaleConfigList = scaleConfigList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        scaleConfigService.saveOrUpdateBatch(scaleConfigList);
        return true;
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, List<Long>>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, List<Long>>> retMap = new HashMap<>();
        QueryWrapper<ScaleConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<ScaleConfig> records = scaleConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<ScaleConfig>> configMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<ScaleConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                Map<String, List<ScaleConfig>> subMap = EntityUtil.makeEntityListMap(entry.getValue(), "uniqueName");
                Map<String, List<Long>> subIdMap = new HashMap<>();
                for (Map.Entry<String, List<ScaleConfig>> subEntry : subMap.entrySet()) {
                    subIdMap.put(subEntry.getKey(), subEntry.getValue().stream().map(i -> i.getId()).distinct().collect(Collectors.toList()));
                }
                retMap.put(entry.getKey(), subIdMap);
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, List<Long>>> getUniqueConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, List<Long>>> retMap = new HashMap<>();
        QueryWrapper<ScaleConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<ScaleConfig> records = scaleConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<ScaleConfig>> configMap = EntityUtil.makeEntityListMap(records, "uniqueName");
        for (Map.Entry<String, List<ScaleConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                Map<String, List<ScaleConfig>> subMap = EntityUtil.makeEntityListMap(entry.getValue(), "hisName");
                Map<String, List<Long>> subIdMap = new HashMap<>();
                for (Map.Entry<String, List<ScaleConfig>> subEntry : subMap.entrySet()) {
                    subIdMap.put(subEntry.getKey(), subEntry.getValue().stream().map(i -> i.getId()).distinct().collect(Collectors.toList()));
                }
                retMap.put(entry.getKey(), subIdMap);
            }
        }
        return retMap;
    }

    /**
     * 数据导出
     *
     * @param response
     * @param hospitalIdVO
     */
    public void exportExcel(HttpServletResponse response, HospitalIdVO hospitalIdVO) {
        QueryWrapper<ScaleConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalIdVO.getHospitalId())
                .orderByDesc("gmt_modified");
        List<ScaleConfig> records = scaleConfigService.list(queryWrapper);
        String fileName = "量表映射.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", ScaleConfig.class, fileName, response, 12.8f);
    }

    /**
     * 各医院映射关系数列表
     *
     * @param hosRelationNumPageVO
     * @return
     */
    public IPage<HosRelationNumDTO> getRelationNumPage(HosRelationNumPageVO hosRelationNumPageVO) {
        return scaleConfigService.getRelationNumPage(hosRelationNumPageVO);
    }

    /**
     * 数据导入模板导出
     *
     * @param response
     */
    public void exportExcelModule(HttpServletResponse response) {
        String fileName = "量表映射模板.xls";
        ExcelUtils.exportExcel(new ArrayList<>(), null, "sheet1", ScaleConfig.class, fileName, response, 12.8f);
    }

    /**
     * 导入数据预匹配
     *
     * @param file
     * @param response
     */
    public void precDataMatch(MultipartFile file, HttpServletResponse response) {
        List<ScaleConfig> originList = ExcelUtils.importExcel(file, 0, 1, ScaleConfig.class);
        List<ScaleConfig> retList = dataProcess(originList);

        String fileName = "量表关联数据（预匹配）.xls";
        ExcelUtils.exportExcel(retList, null, "sheet1", ScaleConfig.class, fileName, response, 12.8f);
    }

    /**
     * 导入数据验证
     *
     * @param file
     * @return
     */
    public Boolean dataVerify(MultipartFile file) {
        List<ScaleConfig> originList = ExcelUtils.importExcel(file, 0, 1, ScaleConfig.class);
        List<ScaleConfig> retList = dataProcess(originList);
        return true;
    }

    /**
     * 数据处理
     *
     * @param originList
     * @return
     */
    public List<ScaleConfig> dataProcess(List<ScaleConfig> originList) {
        List<ScaleConfig> retList = Lists.newLinkedList();
        List<String> hisNameList = originList.stream().map(i -> i.getHisName()).distinct().collect(Collectors.toList());
        Map<String, List<ScaleConfig>> allMap = getAll(hisNameList);

        //去除空格
        originList.forEach(item -> {
            item.setHisName(item.getHisName().trim());
        });

        //获取标准术语
        List<String> precUniqueName = Lists.newArrayList();
        if (allMap != null) {
            for (Map.Entry<String, List<ScaleConfig>> entry : allMap.entrySet()) {
                if (ListUtil.isNotEmpty(entry.getValue())) {
                    precUniqueName.addAll(entry.getValue().stream().map(i -> i.getUniqueName()).collect(Collectors.toList()));
                }
            }
        }
        precUniqueName = precUniqueName.stream().distinct().collect(Collectors.toList());

        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setNames(precUniqueName);
        conceptVO.setType(ConceptTypeEnum.Scale.getKey());
        RespDTO<List<String>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        List<String> uniqueNames = respDTO.data;
        if (ListUtil.isNotEmpty(originList)) {
            for (ScaleConfig originItem : originList) {
                if (allMap.containsKey(originItem.getHisName())) {
                    List<ScaleConfig> items = allMap.get(originItem.getHisName());
                    boolean flag = false;
                    for (ScaleConfig item : items) {
                        if (uniqueNames.contains(item.getUniqueName())) {
                            retList.add(item);
                            flag = true;
                        }
                    }
                    if (!flag) {
                        retList.add(originItem);
                    }
                } else {
                    retList.add(originItem);
                }
            }
        }

        retList = retList.stream()
                .distinct()
                .collect(Collectors.toList());
        return retList;
    }

    /**
     * 获取所有医院映射数据
     *
     * @return
     */
    public Map<String, List<ScaleConfig>> getAll(List<String> hisNameList) {
        Map<String, List<ScaleConfig>> retMap = new HashMap<>();
        QueryWrapper<ScaleConfig> queryWrapper = new QueryWrapper<>();
        if (ListUtil.isNotEmpty(hisNameList)) {
            queryWrapper.in("his_name", hisNameList);
        }
        List<ScaleConfig> records = scaleConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        records.forEach(record -> {
            record.setHospitalId(null);
            record.setId(null);
            record.setUniqueCode(StringUtils.isBlank(record.getUniqueCode()) ? "" : record.getUniqueCode());
        });

        records = records
                .stream()
                .filter(record -> record.getIsDeleted().equals(IsDeleteEnum.N.getKey()))
                .distinct()
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }

        retMap = EntityUtil.makeEntityListMap(records, "hisName");

        return retMap;
    }

    /**
     * 查找指定医院映射关系
     *
     * @param hospitalId
     * @return
     */
    public List<ScaleConfig> getListByHospitalId(Long hospitalId) {
        QueryWrapper<ScaleConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        return scaleConfigService.list(queryWrapper);
    }
}